//
// Chiptune Mega Mega
// Yunji Fengshi (yf88) & Scott Zhao (sz296)
// 

#define F_CPU 16000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <avr/pgmspace.h>
#include <inttypes.h>


#include "lcd_lib.h" 

const int8_t LCD_initialize[] PROGMEM ="\0";
int8_t lcd_buffer[17];

volatile int averageAin, printtimer, prev,max,min,Ain,counter,delta,average,lastAin,nofingercount;
volatile uint16_t led_timer = 500;

float HeartRate;
float tempHR, HR1, HR2, HR3, HR4, HR5, HR6, HR7, HR8, HR9, HR10, HR11, HR12, HR13, HR14, HR15, HR16, HR17, HR18, HR19, HR20;



// Program Functions
void init_lcd(void);	

int main()
{
	// Initialize ports and ADC
	DDRD |= (1 << PORTD2); // set on-board LED to output
	
	ADMUX= (0<<REFS1)|(1<<REFS0)|(1<<ADLAR); 
	ADCSRA=(1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)| (1<<ADATE); //Prescalar 128
	ADCSRB= (0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0); 
	ADCSRA |= 1<<ADSC; //start conversion	
	
	/*
	ADMUX = (1 << REFS0) | (1 << ADLAR); ;  
    
   	//enable ADC and set prescaler to 1/128*16MHz=125,000
  	//and clear interupt enable
   	//and start a conversion
   	ADCSRA = (1<<ADEN) | (1<<ADSC) + 7 ;
	*/	

	// Initialize LCD
	init_lcd();

	// Intialize Timer0 (1-millisecond utility timer)
TCCR1B |= (1 << WGM12) | (1 << CS12) | (0 << CS11) | (1 << CS10); //prescaler = 1024 	TCNT1 = 0; // init counter 1
	OCR1A = 156; // compare value
	TIMSK1 |= (1 << OCIE1A); // compare enable

	
	averageAin = 125;
	average=60; //average values intialized
	HR1 = average;
	HR2 = average;
	HR3 = average;
	HR4 = average;
	HR5 = average;
	HR6 = average;
	HR7 = average;
	HR8 = average;
	HR9 = average;
	HR10 = average;
	HR11 = average;
	HR12 = average;
	HR13 = average;
	HR14 = average;
	HR15 = average;
	HR16 = average;
	HR17 = average;
	HR18 = average;
	HR19 = average;
	HR20 = average;

	nofingercount = 6; // start nofingercount above threshold

	max = 0; //max and mins
	min = 255; 

	sei(); //enable interrupts

	LCDclr(); //clear lcd
	sprintf(lcd_buffer,"No Pulse       ");
	LCDGotoXY(0, 0); 
	LCDstring(lcd_buffer, strlen(lcd_buffer));

	while (1)
	{
		// Calculate heart rate
		tempHR = 6000 / delta;
	
		// Average heart rate over 20 pulses
		if (tempHR < 200 && tempHR > 20)	
		{
			HR20 = HR19;
			HR19 = HR18;
			HR18 = HR17;
			HR17 = HR16;
			HR16 = HR15;
			HR15 = HR14;
			HR14 = HR13;
			HR13 = HR12;
			HR12 = HR11;
			HR11 = HR10;
			HR10 = HR9;
			HR9 = HR8;
			HR8 = HR7;
			HR7 = HR6;
			HR6 = HR5;
			HR5 = HR4;
			HR4 = HR3;
			HR3 = HR2;
			HR2 = HR1;
			HR1 = tempHR;
		
			HeartRate = (HR1+HR2+HR3+HR4+HR5+HR6+HR7+HR8+HR9+HR10+HR11+HR12+HR13+HR14+HR15+HR16+HR17+HR18+HR19+HR20)/20;
		}	

		// If Ain value is not changing, then there is no finger present
		if (prev < (Ain + 5) && prev > (Ain - 5)) {
			nofingercount++;
		}
		else {
			nofingercount = 0;
		}

		if (nofingercount > 5) {
			HeartRate = 0;
			sprintf(lcd_buffer,"No Pulse       ");
		}
		
		else{ 
			sprintf(lcd_buffer,"Heart Rate=%3.0f ",HeartRate);
		}
	

		// Update LCD and write to PORTA
		if (printtimer > 1000)
		{
			// Reset print timer
			printtimer = 0;

						// Output to PortB
			PORTB = HeartRate;

			LCDGotoXY(0, 0);

			LCDstring(lcd_buffer, strlen(lcd_buffer));
		}

		// Output Ain values to lcd for debugging
		sprintf(lcd_buffer,"Ain=%d    ",Ain);
		LCDGotoXY(0, 1); 
		LCDstring(lcd_buffer, strlen(lcd_buffer));

		// Save current Ain value for determining whether or not a finger is present
		// prev = Ain;

	} // end while

} // end main

ISR(TIMER1_COMPA_vect)
{

	counter++; // increment
	printtimer++;

	//read Ain and swap Ain value with previous value
	lastAin = Ain;
	Ain = ADCH; 

	if (Ain < min)
	{
		min = Ain; 
	}
	else if(Ain > max)
	{
		max = Ain;
	}

	// Detect pulse if threshold is crossed
	if( lastAin > averageAin && Ain < averageAin && (max-min) > 40 )
	{
		PORTD ^= (1 << PORTD2);
		averageAin = ((max+min)/2); // set threshold for detecting next pulse
		delta = counter; // save time between pulses
		counter = 0; // reset counter
		max = 0; 
		min = 255;
	}

}

void init_lcd(void)
{
	LCDinit(); 
	LCDcursorOFF();
	LCDclr(); 
	LCDGotoXY(0,0); // go to x and y position on the lcd
}
